package org.artifactory.queue;

import org.artifactory.storage.event.EventInfo;

import java.util.Set;


/**
 * <p>
 * If implemented - allows a specific event to take part in different groups (keys).<br>
 * each group will run in parallel.<br>
 * <br>
 * There are 2 main use cases here:<br>
 * <br>
 * 1. Dividing events into parallel running groups based on event path <br>
 * <pre>
 * events "create|repo/a/a1/c1.txt","create|repo/a/a2/c2.txt"
 * keys.add(e.getRepoPath().toPath());
 * </pre>
 * 2. Duplicating a specific event into several groups(e.g: debian's distribution)<br>
 * <pre>
 * event "create|myLocalDebianRepo/dist/1.deb"
 * keys.add("myLocalDebianRepo/dist1");
 * keys.add("myLocalDebianRepo/dist2");
 * keys.add("myLocalDebianRepo/dist3");
 * </pre>
 *
 * If not implemented - there will be only one group of events. key=repoKey<br> *
 *
 * @author avishayh
 */
@FunctionalInterface
public interface QueueEventAggregator {

    void setKeys(EventInfo event, Set<String> keys);

}
