package org.artifactory.queue.behaviors;

import org.artifactory.storage.event.EventInfo;
import org.artifactory.storage.event.EventType;

import java.util.List;

/**
 * A chain of reduction behaviors to be used for each of the use cases of PersistentQueue
 * Each implementation can configure different behavior for their needs.
 *
 * If not chosen differently Basic is the default behavior (see QueueConfiguration.reductionBehaviors)
 *
 * @author avishayh
 */
public enum QueueReductionBehaviours {
    None(),
    Basic(
            new CreateEliminateDeleteBehavior(),
            new DeleteEliminateUpdateOrCreateBehavior(), //must be after CreateEliminateDelete (selfElimination)
            new PrioritizedEventDedupBehavior(EventType.create, EventType.update)
    ),
    FederatedReplication(
            new FederatedDedupBehavior()
    );

    private final EventReductionBehavior[] chain;

    QueueReductionBehaviours(EventReductionBehavior... chain) {
        this.chain = chain;
    }

    public List<EventInfo> applyChain(List<EventInfo> events) {
        return applyChain(chain, events);
    }

    public static List<EventInfo> applyChain(EventReductionBehavior[] chain, List<EventInfo> events) {
        List<EventInfo> result = events;
        for (EventReductionBehavior behavior : chain) {
            result = behavior.apply(result);
        }
        return result;
    }
}
