package org.artifactory.storage.db.event.service;

import org.artifactory.storage.db.event.dao.EventsDao;
import org.artifactory.storage.db.event.dao.NodeEventsCursorDao;
import org.artifactory.storage.db.event.entity.DbNodeEventCursor;
import org.artifactory.storage.db.event.entity.NodeEventCursorType;
import org.artifactory.storage.db.event.model.NodeEventCursor;
import org.jfrog.common.ClockUtils;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.sql.SQLException;
import java.util.Optional;

import static org.artifactory.storage.db.event.entity.NodeEventCursorType.getEventIdLogTypes;
import static org.artifactory.storage.db.event.entity.NodeEventCursorType.getTimestampLogTypes;
import static org.fest.assertions.Assertions.assertThat;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * @author Uriah Levy
 */
@Test
public class NodeEventCursorServiceImplTest {

    private static final String METADATA_OPERATOR_ID = "metadata-dispatcher";

    private NodeEventCursorService service;
    private NodeEventsCursorDao nodeEventsCursorDao;
    private EventsDao eventsDao;

    @BeforeMethod
    public void setup() {
        eventsDao = mock(EventsDao.class);
        nodeEventsCursorDao = mock(NodeEventsCursorDao.class);
        service = new NodeEventCursorServiceImpl(nodeEventsCursorDao, eventsDao);
    }

    public void updateCursor() throws SQLException {
        long eventTimestamp = ClockUtils.epochMillis();
        service.updateOrInsertCursor(
                new NodeEventCursor(METADATA_OPERATOR_ID, eventTimestamp, NodeEventCursorType.METADATA_MIGRATION));
        DbNodeEventCursor eventCursor = DbNodeEventCursor.builder()
                .eventMarker(eventTimestamp)
                .operatorId(METADATA_OPERATOR_ID)
                .type(NodeEventCursorType.METADATA_MIGRATION)
                .build();
        verify(nodeEventsCursorDao).update(eventCursor);
    }

    public void cursorForOperatorId() throws SQLException {
        long eventTimestamp = ClockUtils.epochMillis();
        DbNodeEventCursor dbNodeEventCursor = DbNodeEventCursor.builder()
                .eventMarker(eventTimestamp)
                .operatorId(METADATA_OPERATOR_ID)
                .type(NodeEventCursorType.METADATA_MIGRATION)
                .build();
        when(nodeEventsCursorDao.cursorForOperator(METADATA_OPERATOR_ID)).thenReturn(Optional.of(dbNodeEventCursor));
        assertEquals(service.cursorForOperator(METADATA_OPERATOR_ID),
                new NodeEventCursor(METADATA_OPERATOR_ID, eventTimestamp, NodeEventCursorType.METADATA_MIGRATION));
    }

    @Test(dataProvider = "provideTimestamps")
    public void oldestTimestampForEventLog(long oldestTimestamp, long oldestEventId, long eventIdTimestamp,
            long expected) throws SQLException {
        when(nodeEventsCursorDao.oldestMarkerForTypes(getTimestampLogTypes())).thenReturn(oldestTimestamp);
        when(nodeEventsCursorDao.oldestMarkerForTypes(getEventIdLogTypes())).thenReturn(oldestEventId);
        when(eventsDao.getOldestTimestampByEventIdInclusive(oldestEventId)).thenReturn(eventIdTimestamp);
        assertThat(service.oldestTimestampForEventLogOperator()).isEqualTo(expected);
    }

    @DataProvider
    private Object[][] provideTimestamps() {
        return new Object[][] {
                {100000L, 100, 50000L, 50000L},
                {100000L, 100, 200000L, 100000L},
                {100000L, 0, -1L, 100000L},
        };
    }
}